@@ -83,14 +83,6 @@ _hx509_cert_private_key (hx509_cert /*p*/);
 int
 _hx509_cert_private_key_exportable (hx509_cert /*p*/);
 
-int
-_hx509_cert_public_encrypt (
-	hx509_context /*context*/,
-	const heim_octet_string */*cleartext*/,
-	const hx509_cert /*p*/,
-	heim_oid */*encryption_oid*/,
-	heim_octet_string */*ciphertext*/);
-
 void
 _hx509_cert_set_release (
 	hx509_cert /*cert*/,
@@ -201,6 +193,9 @@ _hx509_find_extension_subject_key_id (
 	const Certificate */*issuer*/,
 	SubjectKeyIdentifier */*si*/);
 
+const struct signature_alg *
+_hx509_find_sig_alg (const heim_oid */*oid*/);
+
 int
 _hx509_generate_private_key (
 	hx509_context /*context*/,
@@ -309,14 +304,6 @@ _hx509_name_modify (
 	const heim_oid */*oid*/,
 	const char */*str*/);
 
-int
-_hx509_parse_private_key (
-	hx509_context /*context*/,
-	const AlgorithmIdentifier */*keyai*/,
-	const void */*data*/,
-	size_t /*len*/,
-	hx509_private_key */*private_key*/);
-
 int
 _hx509_path_append (
 	hx509_context /*context*/,
@@ -349,55 +336,31 @@ _hx509_pi_printf (
 	const char */*fmt*/,
 	...);
 
-int
-_hx509_private_key2SPKI (
-	hx509_context /*context*/,
-	hx509_private_key /*private_key*/,
-	SubjectPublicKeyInfo */*spki*/);
-
 void
-_hx509_private_key_assign_rsa (
-	hx509_private_key /*key*/,
-	void */*ptr*/);
+_hx509_private_eckey_free (void */*eckey*/);
 
 int
 _hx509_private_key_export (
 	hx509_context /*context*/,
 	const hx509_private_key /*key*/,
+	hx509_key_format_t /*format*/,
 	heim_octet_string */*data*/);
 
 int
 _hx509_private_key_exportable (hx509_private_key /*key*/);
 
-int
-_hx509_private_key_free (hx509_private_key */*key*/);
-
 BIGNUM *
 _hx509_private_key_get_internal (
 	hx509_context /*context*/,
 	hx509_private_key /*key*/,
 	const char */*type*/);
 
-int
-_hx509_private_key_init (
-	hx509_private_key */*key*/,
-	hx509_private_key_ops */*ops*/,
-	void */*keydata*/);
-
 int
 _hx509_private_key_oid (
 	hx509_context /*context*/,
 	const hx509_private_key /*key*/,
 	heim_oid */*data*/);
 
-int
-_hx509_private_key_private_decrypt (
-	hx509_context /*context*/,
-	const heim_octet_string */*ciphertext*/,
-	const heim_oid */*encryption_oid*/,
-	hx509_private_key /*p*/,
-	heim_octet_string */*cleartext*/);
-
 hx509_private_key
 _hx509_private_key_ref (hx509_private_key /*key*/);
 
@@ -445,26 +408,6 @@ _hx509_request_add_email (
 	hx509_request /*req*/,
 	const char */*email*/);
 
-void
-_hx509_request_free (hx509_request */*req*/);
-
-int
-_hx509_request_get_SubjectPublicKeyInfo (
-	hx509_context /*context*/,
-	hx509_request /*req*/,
-	SubjectPublicKeyInfo */*key*/);
-
-int
-_hx509_request_get_name (
-	hx509_context /*context*/,
-	hx509_request /*req*/,
-	hx509_name */*name*/);
-
-int
-_hx509_request_init (
-	hx509_context /*context*/,
-	hx509_request */*req*/);
-
 int
 _hx509_request_parse (
 	hx509_context /*context*/,
@@ -477,18 +420,6 @@ _hx509_request_print (
 	hx509_request /*req*/,
 	FILE */*f*/);
 
-int
-_hx509_request_set_SubjectPublicKeyInfo (
-	hx509_context /*context*/,
-	hx509_request /*req*/,
-	const SubjectPublicKeyInfo */*key*/);
-
-int
-_hx509_request_set_name (
-	hx509_context /*context*/,
-	hx509_request /*req*/,
-	hx509_name /*name*/);
-
 int
 _hx509_request_to_pkcs10 (
 	hx509_context /*context*/,
@@ -500,7 +431,7 @@ hx509_revoke_ctx
 _hx509_revoke_ref (hx509_revoke_ctx /*ctx*/);
 
 void
-_hx509_sel_yyerror (char */*s*/);
+_hx509_sel_yyerror (const char */*s*/);
 
 int
 _hx509_self_signed_valid (
@@ -515,10 +446,16 @@ _hx509_set_cert_attribute (
 	const heim_octet_string */*attr*/);
 
 int
-_hx509_signature_best_before (
+_hx509_set_digest_alg (
+	DigestAlgorithmIdentifier */*id*/,
+	const heim_oid */*oid*/,
+	const void */*param*/,
+	size_t /*length*/);
+
+int
+_hx509_signature_is_weak (
 	hx509_context /*context*/,
-	const AlgorithmIdentifier */*alg*/,
-	time_t /*t*/);
+	const AlgorithmIdentifier */*alg*/);
 
 void
 _hx509_unmap_file_os (heim_octet_string */*os*/);
